<?php

/**
 * Gives us admin boxes on pages and posts that allow us to tie content to the purchase of a product
 *
 * @since 1.0
 * @author SFNdesign, Curtis McHale
 */
class WECR_Content_Restriction_Meta{

	function __construct(){

		add_action( 'load-post.php', array( $this, 'metaboxes_setup' ) );
		add_action( 'load-post-new.php', array( $this, 'metaboxes_setup' ) );

		add_action( 'save_post', array( $this, 'kill_transient' ) );

	} // __construct

	/**
	 * Adds our actions so that we can start the build out of the post metaboxes
	 *
	 * @since   1.0
	 * @author  SFNdesign, Curtis McHale
	 */
	public function metaboxes_setup(){

		// adds the action which actually adds the meta boxes
		add_action( 'add_meta_boxes', array( $this, 'add_post_metaboxes' ) );

		// all the saving actions go here
		add_action( 'save_post', array( $this, 'save_post_meta' ), 10, 2 );

	} // metaboxes_setup

	/**
	 * Adds the actual metabox
	 *
	 * @uses    add_meta_box
	 *
	 * @since   0.0.1
	 * @author  SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @uses $this->allowed_post_types()        Returns the array of allowed post types for our meta boxes
	 */
	public function add_post_metaboxes(){

		$allowed_post_types = $this->allowed_post_types();

		foreach( $allowed_post_types as $p ){

			add_meta_box(
				'wecr-easy-restrict-content',           // $id - HTML 'id' attribute of the edit screen section
				__( 'Restrict Content', 'wecr' ),       // $title - Title that will show at the top of the metabox
				array( $this, 'display_metaboxes' ),    // $callback - The function that will display the metaboxes
				$p,                                     // $posttype - The registered name of the post type you want to show on
				'side',                                 // $context - Where it shows on the page. Possibilities are 'normal', 'advanced', 'side'
				'default'                               // $priority - How high should this display?
				//'$callback_args'                      // any extra params that the callback should get. It will already get the $post_object
			);

			if ( $p == 'forum' ) continue;
			add_meta_box(
				'wecr-easy-restrict-content-message',           // $id - HTML 'id' attribute of the edit screen section
				__( 'Custom Restricted Message', 'wecr' ),      // $title - Title that will show at the top of the metabox
				array( $this, 'display_message_metaboxes' ),    // $callback - The function that will display the metaboxes
				$p,                                             // $posttype - The registered name of the post type you want to show on
				'normal',                                       // $context - Where it shows on the page. Possibilities are 'normal', 'advanced', 'side'
				'default'                                       // $priority - How high should this display?
				//'$callback_args'                              // any extra params that the callback should get. It will already get the $post_object
			);

		}

	} // add_post_meta

	/**
	 * Returns array of allowed post_types for our metaboxes
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @return array    array of post_types we want to show the metabox on
	 *
	 * @uses Woo_Easy_Content_Restriction->allowed_post_types()     Provides us with the array of allowed post_types
	 */
	private function allowed_post_types(){

		$base = $GLOBALS['wecr'];

		return $base->allowed_post_types();

	} // allowed_post_types

	/**
	 * Builds out the metabox for a custom restricted content message
	 *
	 * @since   0.0.1
	 * @author  SFNdesign, Curtis McHale
	 *
	 * @param object        $object         required    The whole post object for the metabox
	 * @param array         $box            required    Array of the box arguements
	 * @uses wp_nonce_field()                           Creates a nonce in WordPress
	 * @uses get_post_meta()                            Returns post meta given key and post_id
	 * @uses wp_editor()                                Returns the WordPress editor
	 * @uses wp_kses_post()                             Makes post-like content safe
	 */
	public function display_message_metaboxes( $post_object, $box ){

		wp_nonce_field( basename( __FILE__ ), 'wecr_easy_restrict_content_meta_nonce' );

		$message = get_post_meta( absint( $post_object->ID ), '_wecr_custom_message', true );
		$editor_content = ! empty( $message ) ? $message : ' ';

		$checkvalue = get_post_meta( absint( $post_object->ID ), '_wecr_checked', true );
	?>
		<p>
			<label for="wecr_restrict_content_message_check"><?php _e( 'Add a custom message:', 'wecr' ); ?></label>
			<input type="checkbox" name="wecr_restrict_content_message_check" id="wecr_restrict_content_message_check" <?php checked( 'on', $checkvalue ); ?> /><br />
		</p>
		<p id="wecr_restrict_content_message_wrapper">
			<label for="wecr_restrict_content_message"><?php _e( 'Custom message:', 'wecr' ); ?></label>
			<?php wp_editor( wp_kses_post( $editor_content ), 'wecr_restrict_content_message' ); ?>
			<span class="description"><?php _e( "If you want a custom message for users when they view restricted content, this is the place to put it", 'wecr' ); ?>.</span>
		</p>

	<?php
	} // display_metaboxes

	/**
	 * Builds out the photo by metabox
	 *
	 * @since   1.0
	 * @author  SFNdesign, Curtis McHale
	 *
	 * @param object    $object     required                The whole post object for the metabox
	 * @param array     $box        optional                Array of the box arguements
	 * @uses get_post_meta()                                Gets post meta given key and post_id
	 * @uses wp_nonce_field()                               Creates a nonce for use to check
	 * @uses $this->show_menu_restriction_checkbox()        Checkbox that allows user to toggle the menu restriction for pages
	 *
	 */
	public function display_metaboxes( $post_object, $box ){

		wp_nonce_field( basename( __FILE__ ), 'wecr_easy_restrict_content_meta_nonce' );

	?>

		<p>
			<label for="wecr-restricted-product"><?php _e( 'Restrict Content to:', 'wecr' ); ?></label>
			<?php $this->get_product_selectbox( absint( $post_object->ID ) ); ?>
			<span class="description"><?php _e( 'If users have bought any of the above products they will see the content', 'wecr' ); ?></span>
		</p>

		<?php $this->show_menu_restriction_checkbox( $post_object ); ?>

		<?php $this->show_my_account_restriction_checkbox( $post_object ); ?>

	<?php
	} // display_metaboxes

	/**
	 * Showing the metabox to remove pages from my account page
	 *
	 * @since 1.2.4
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param object    $post_object    required        Post object that we are adding a meta box to
	 *
	 * @uses get_post_type()                            Returns post_type string given post_id
	 * @uses get_post_meta()                            Returns post meta given post_id and key
	 * @uses checked()                                  Magic WP function that gives you the checked HTML values (jazz hands)
	 */
	private function show_my_account_restriction_checkbox( $post_object ){

		$allowed_post_types = $this->allowed_meta_post_types();

		if ( ! in_array( get_post_type( $post_object->ID ), $allowed_post_types ) ) return;

		$current_value = get_post_meta( $post_object->ID, '_wecr_remove_from_my_account', true );

	?>

		<p>
			<label for="wecr-restricted-my-account"><?php _e( 'Remove content from My Account', 'wecr' ); ?></label>
			<input type="checkbox" name="wecr-restricted-my-account" id="wecr-restricted-my-account" <?php checked( 'on', $current_value ); ?> /><br />
			<span class="description"><?php _e( "If checked then the page/post won't show up on the My Account list of restricted content", 'wecr' ); ?></span>
		</p>

	<?php
	} // show_menu_restriction_checkbox

	/**
	 * Showing the metabox to remove pages from menus
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param object    $post_object    required        Post object that we are adding a meta box to
	 *
	 * @uses get_post_type()                            Returns post_type string given post_id
	 * @uses get_post_meta()                            Returns post meta given post_id and key
	 * @uses checked()                                  Magic WP function that gives you the checked HTML values (jazz hands)
	 */
	private function show_menu_restriction_checkbox( $post_object ){

		$allowed_post_types = $this->allowed_meta_post_types();

		if ( ! in_array( get_post_type( $post_object->ID ), $allowed_post_types ) ) return;

		$current_value = get_post_meta( $post_object->ID, '_wecr_remove_from_menu', true );

	?>

		<p>
			<label for="wecr-restricted-menu-item"><?php _e( 'Remove content from menu', 'wecr' ); ?></label>
			<input type="checkbox" name="wecr-restricted-menu-item" id="wecr-restricted-menu-item" <?php checked( 'on', $current_value ); ?> /><br />
			<span class="description"><?php _e( "If checking this box makes sure that the pages won't show up in the menu", 'wecr' ); ?></span>
		</p>

	<?php
	} // show_menu_restriction_checkbox

	/**
	 * Gives us a list of the post_types allowed to show our meta extra options
	 *
	 * @since 1.2.6
	 * @author SFNdesign, Curtis McHale
	 *
	 * @return array        $types          The array of types that are allowed
	 */
	private function allowed_meta_post_types(){

		$types = array(
			'page',
		);

		return apply_filters( 'wecr_allowed_meta_post_types', $types );
	}

	/**
	 * Builds as select box of all the products that are possible for the content
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @uses $this->get_products()          Returns array of products
	 * @uses get_post_meta()                Gets metadata for post given key and value
	 * @uses selected()                     Returns selected given 2 values to compare
	 * @uses get_the_title()                Returns the title of the post given post_id
	 * @uses $this->get_variations()        Returns array that may have variations in it
	 */
	private function get_product_selectbox( $post_id ){

		$products = $this->get_products();
		$value = get_post_meta( absint( $post_id ), '_wecr_easy_restrict_content', true );
		
		if ( empty( $value ) ){
			$value = array();
		}
	?>

		<select name="wecr-restricted-product[]" id="wecr-restricted-product" class="chosen" multiple>
			<option value=""><?php _e( 'None', 'wecr' ); ?></option>

			<?php foreach( $products as $p ) {
				$selected = in_array( $p->ID, $value ) ? 'selected="selected"' : '';
				$variations = $this->get_variations( $p->ID );
			?>

				<?php if ( empty( $variations ) ){ ?>

					<option value="<?php echo (int) $p->ID; ?>" <?php echo $selected; ?>><?php echo get_the_title( absint( $p->ID ) ); ?></option>

				<?php } else {
					$this->get_variation_optgroup( $p->ID, $variations, $value );
				} // if variations

			 } // foreach products ?>
		</select><br />
	<?php
	} // get_product_selectbox

	/**
	 * Gives us an optgroup for variations
	 *
	 * @since 1.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param int       $product_id         required            The Product id we need variations for
	 * @param array     $variations         required            Array of variations
	 * @param int       $selected_value     required            Int that is 'selected' or saved as the restriction case
	 * @uses get_the_title()                                    Returns the title given post_id
	 * @uses absint()                                           Makes sure we don't have a negative number
	 * @uses WP_Product_Variation()                             WooCommerce product variation class
	 * @uses selected()                                         WP magic function for dealing with selected HTML
	 * @uses WP_Product_Variation->get_formatted_name()         Returns the formatted name of a variation
	 */
	private function get_variation_optgroup( $product_id, $variations, $selected_value ){ ?>

		<optgroup label="<?php echo get_the_title( absint( $product_id ) ); ?>">
			<?php foreach( $variations as $v ){
				$vp = new WC_Product_Variation( $v->ID );
			?>
				<?php $vselected = in_array( $v->ID, $selected_value ) ? 'selected="selected"' : ''; ?>
				<option value="<?php echo (int) $v->ID; ?>" <?php echo $vselected; ?>><?php echo $vp->get_formatted_name(); ?></option>
			<?php } ?>
		</optgroup>

	<?php
	} // get_variations_optgroup

	/**
	 * Returns variations given a product_id
	 *
	 * @since 1.1
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param int       $product_id             required        The id of the product we need variations for
	 * @uses get_posts()                                        returns posts given args
	 * @uses absint()                                           Makes sure I don't have a negative number
	 * @return array    $variations                             Array of variations
	 */
	private function get_variations( $product_id ){

		$variations = get_posts( array( 'post_type' => 'product_variation', 'post_parent' => absint( $product_id ) ) );

		return $variations;

	} // get variations

	/**
	 * Returns array of products
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @uses get_posts()       Returns posts based on $args provided
	 */
	public function get_products(){

		$products = get_transient( 'wecr_easy_restrict_products' );

		if ( false === $products ){

			$args = array(
				'post_type'      => 'product',
				'posts_per_page' => -1,
			);

			$products = get_posts( $args );

			set_transient( 'wecr_easy_restrict_products', $products, 24 * 60 );

		} // if $products === false

		return $products;

	} // get_products

	/**
	 * Destroys the transient of products when we add another one
	 *
	 * Notice that after it kills it we rebuild the transient on the shutdown hook.
	 * This means that it will be built for our users when they visit the page and they
	 * don't have to sit and wait for it to build the first time.
	 *
	 * @since 0.0.1
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @param int   $post_id    required        The post_id that is currently being saved
	 *
	 * @uses get_post_type()                    Returns the registered name of a post type given the post_id
	 * @uses delete_transient()                 Deletes the transient from the site
	 */
	public function kill_transient( $post_id ){

		if ( get_post_type( $post_id ) === 'product' ){
			delete_transient( 'wecr_easy_restrict_products' );
			add_action( 'shutdown', array( $this, 'get_products' ) );
		}

	} // kill_transient

	/**
	 * Saves the metaboxes
	 *
	 * @param   int     $post_id    req     The ID of the post we're saving8
	 * @param   object  $post       req     The whole post object
	 * @return  mixed
	 *
	 * @uses    wp_verify_nonce
	 * @uses    get_post_type_object
	 * @uses    delete_post_meta
	 * @uses    update_post_meta
	 *
	 * @since   1.0
	 * @author  SFNdesign, Curtis McHale
	 */
	public function save_post_meta( $post_id, $post ){

		// check the nonce before we do any processing
		if( ! isset( $_POST[ 'wecr_easy_restrict_content_meta_nonce' ] ) || ! wp_verify_nonce( $_POST[ 'wecr_easy_restrict_content_meta_nonce' ], basename( __FILE__ ) ) ) return $post_id;

		// get the post type object
		$post_type = get_post_type_object( $post->post_type );

		// make sure current user can save
		if( ! current_user_can( $post_type->cap->edit_post, $post_id ) ) return $post_id;

		$product_id         = empty( $_POST['wecr-restricted-product'] ) ? delete_post_meta( $post_id, '_wecr_easy_restrict_content' ) : update_post_meta( $post_id, '_wecr_easy_restrict_content', $_POST['wecr-restricted-product'] );
		$message            = empty( $_POST['wecr_restrict_content_message'] ) ? delete_post_meta( $post_id, '_wecr_custom_message' ) : update_post_meta( $post_id, '_wecr_custom_message', wp_kses_post( $_POST['wecr_restrict_content_message'] ) );
		$checked            = empty( $_POST['wecr_restrict_content_message_check'] ) ? delete_post_meta( $post_id, '_wecr_checked' ) : update_post_meta( $post_id, '_wecr_checked', esc_attr( $_POST['wecr_restrict_content_message_check'] ) );
		$restricted         = empty( $_POST['wecr-restricted-menu-item'] ) ? delete_post_meta( $post_id, '_wecr_remove_from_menu' ) : update_post_meta( $post_id, '_wecr_remove_from_menu', esc_attr( $_POST['wecr-restricted-menu-item'] ) );
		$restricted_account = empty( $_POST['wecr-restricted-my-account'] ) ? delete_post_meta( $post_id, '_wecr_remove_from_my_account' ) : update_post_meta( $post_id, '_wecr_remove_from_my_account', esc_attr( $_POST['wecr-restricted-my-account'] ) );

	} // save_post_meta

} // WECR_Content_Restriction_Meta

$GLOBALS['wecr_meta'] = new WECR_Content_Restriction_Meta();
